;*******************************************************************************
;*                                                                             *
;* Programm: Test_Timer1.asm                                                       *
;* Version:  0.1                                                               *
;*                                                                             *
;* Dieses Programm dient zum Test des Timer/Counter1                           *
;*                                                                             *
;* Mikroprozessor:          ATmega128                                          *
;* Oszillatorfrequenz:      14,745600 MHz (externer Quarzoszillator)           *
;* bertragungsrate USART0: 115,200 kbps                                       *
;*                                                                             *
;*******************************************************************************

;-------------------------------------------------------------------------------
; Definitionsdatei des Mikroprozessors einbinden
;-------------------------------------------------------------------------------
.nolist
.include	"m128def.inc"
.list


;-------------------------------------------------------------------------------
; Symbol-Definitionen (Variablen)
;-------------------------------------------------------------------------------
.def	tmp1		= r16
.def	tmp2		= r17
.def	SREG_save	= r18		; Register zum sichern des SREG
.def	Rx_Flg	= r19		; Status-Flag fr den Datenempfang


;-------------------------------------------------------------------------------
; Symbol-Definitionen (Konstanten)
;-------------------------------------------------------------------------------
; Verwendete Symbole fr den USART0:
.equ	CLOCK	= 14745600		; Oszillatorfrequenz (in Hz)
.equ	BAUD	= 115200		; UART0: Baudrate (in bps)
.equ	S	= 16			; UART0: Asyncronous Normal Speed Mode, U2X=0
.equ	UBRR0	= CLOCK/(BAUD*S)-1; UART0: Berechnung des Wertes fr die USART
					;   Baud Rate Registers 0 (UBRR0H und UBRR0L)
.equ	TXLED	= 5			; Die TX-LED wird von PortE5 (PE5) angesteuert
.equ	RXLED	= 4			; Die RX-LED wird von PortE4 (PE4) angesteuert

; Verwendete Adressen im SRAM:
.equ	TSymb_H	= $0100	; Adresse im SRAM fr die Symboldauer (High-Byte)
.equ	TSymb_L	= $0101	; Adresse im SRAM fr die Symboldauer (Low-Byte)



;-------------------------------------------------------------------------------
; Speicherorganisation / Interruptvektorentabelle (vgl. Datenblatt S. 59)
;
; Alle Einsprungadressen der nicht benutzten Interrupts werden sicherheitshalber
; mit einem "Return from Interrupt" (reti) geschlossen. 
;-------------------------------------------------------------------------------
.cseg
.org	0x0000
	jmp	RESET		;$0000: RESET Handler = Programmstart
	; ACHTUNG: Bei Verwendung von "jmp" KEIN "nop" einfgen!!!
	reti			;$0002: External Interrupt0 Vector Address
	nop
	reti			;$0004: External Interrupt1 Vector Address
	nop
	reti			;$0006: External Interrupt2 Vector Address
	nop
	reti			;$0008: External Interrupt3 Vector Address
	nop
	reti			;$000a: External Interrupt4 Vector Address
	nop
	reti			;$000c: External Interrupt5 Vector Address
	nop
	reti			;$000e: External Interrupt6 Vector Address
	nop
	reti			;$0010: External Interrupt7 Vector Address
	nop
	reti			;$0012: Output Compare2 Interrupt Vector Address
	nop
	reti			;$0014: Overflow2 Interrupt Vector Address
	nop
	reti			;$0016: Input Capture1 Interrupt Vector Address
	nop
	jmp	TIM1_COMPA	;$0018: Output Compare1A Interrupt Vector Address
	; ACHTUNG: Bei Verwendung von "jmp" KEIN "nop" einfgen!!!
	reti			;$001a: Output Compare1B Interrupt Vector Address
	nop
	reti			;$001c: Overflow1 Interrupt Vector Address
	nop
	reti			;$001e: Output Compare0 Interrupt Vector Address
	nop
	reti			;$0020: Overflow0 Interrupt Vector Address
	nop
	reti			;$0022: SPI Interrupt Vector Address
	nop
	jmp	USART0_RXC	;$0024: USART0 Receive Complete Interrupt Vector Address
	; ACHTUNG: Bei Verwendung von "jmp" KEIN "nop" einfgen!!!
	reti			;$0026: USART0 Data Register Empty Interrupt Vector Add.
	nop
	reti			;$0028: USART0 Transmit Complete Interrupt Vector Add.
	nop
	reti			;$002a: ADC Conversion Complete Handle
	nop
	reti			;$002c: EEPROM Write Complete Handle
	nop
	reti			;$002e: Analog Comparator Interrupt Vector Address
	nop
	reti			;$0030: Output Compare1C Interrupt Vector Address
	nop
	reti			;$0032: Input Capture3 Interrupt Vector Address
	nop
	reti			;$0034: Output Compare3A Interrupt Vector Address
	nop
	reti			;$0036: Output Compare3B Interrupt Vector Address
	nop
	reti			;$0038: Output Compare3C Interrupt Vector Address
	nop
	reti			;$003A: Overflow3 Interrupt Vector Address
	nop
	reti			;$003C: USART1 Receive Complete Interrupt Vector Address
	nop
	reti			;$003E: USART1 Data Register Empty Interrupt Vector Add.
	nop
	reti			;$0040: USART1 Transmit Complete Interrupt Vector Add.
	nop
	reti			;$0042: TWI Interrupt Vector Address
	nop
	reti			;$0044: Store Program Memory Ready Interrupt Vector Add.
	nop


;*******************************************************************************
;*                                                                             *
;* Hauptprogramm:                                                              *
;*                                                                             *
;* Zweck:                                                                      *
;*                                                                             *
;*******************************************************************************

;.org		0x100
RESET:	ldi	tmp1,LOW(RAMEND)	; Initialisierung des Stack-Pointers
		out	SPL,tmp1		; 
		ldi	tmp1,HIGH(RAMEND)	;
		out	SPH,tmp1		; 
		call	Init_IO		; Initialisierung der Ein- und Ausgabeports
		sei				; Interrupts global freigeben


MAIN:		call	Test_SRAM		
		call	TC1_Start		

LOOP:		nop				; Endlosschleife: Warte auf einen Interrupt
		rjmp	LOOP

;-------------------------------------------------------------------------------
; Unterprogramme des Hauptprogramms
;-------------------------------------------------------------------------------

Init_IO:	ser	tmp1			; Setze alle Bits in Register tmp1
		out	DDRA,tmp1		; PortA ist Ausgang (High-Byte des DDS)
		out	DDRC,tmp1		; PortC ist Ausgang (Low-Byte des DDS)
		out	DDRD,tmp1		; PortD ist Ausgang (Steuereingnge des DDS)
		sbi	DDRE,RXLED		; PortE4 ist Ausgang (fr RX-LED)
		sbi	DDRE,TXLED		; PortE5 ist Ausgang (fr TX-LED)
		ret

Test_SRAM:	; Hex-Werte in den Registerdatenbereich schreiben:
		ldi	tmp1,0x01		; Timerwert: 01 71
		sts	$0100,tmp1		;   ==> 200 s
		ldi	tmp1,0x71
		sts	$0101,tmp1
		nop	; Programm gekrzt !!!
		ret

;*******************************************************************************
;*                                                                             *
;* Interruptroutine: "USART0_RXC"  (USART0 RX Complete Handler)                *
;*                                                                             *
;* Zweck: Diese Routine wird ausgefhrt sobald ein Byte ber das USART0        *
;*        empfangen wurde.                                                     *
;*        Nach dem Empfang des letzten Bytes wird von dieser Interruptroutine  *
;*        der Timer/Counter1 initialisiert und gestartet.                      *
;*                                                                             *
;*******************************************************************************

USART0_RXC:	nop				; Programm gekrzt !!!
		reti


;-------------------------------------------------------------------------------
; Unterprogramme: "TC1_Start" und "TC1_Ende"
; 
; Zweck: Timer zur zeitlichen Steuerung der Modulation (DDS)
; 
; Diese Unterprogramme werden nur von der Interruptroutine "USART0_RXC" 
; verwendet.
;-------------------------------------------------------------------------------

TC1_Start:	in	SREG_save,SREG	; SREG (und damit das I-Flag) sichern
		cli				; Interrupts deaktivieren (SREG: I-Flag=0)

		; Output Compare Register 1 A (OCR1AH und OCR1AL), vgl. S.137:
		;   ==> Wird mit dem TOP-Wert fr den Timer/Counter1 geladen!
		;   ACHTUNG: ATOMIC-WRITE ntig!
		lds	tmp2,TSymb_L	; Lade Timer-Wert (Low-Byte)
		lds	tmp1,TSymb_H	; Lade Timer-Wert (High-Byte)
		out	OCR1AL,tmp2		; TOP-Wert fr TC1 speichern
		out	OCR1AH,tmp1		;   (ATOMIC-WRITE)

		; Timer/Counter Control Register B (TCCR1B), vgl. S.135:
		in	tmp1,TCCR1B
		sbr	tmp1,(1<<WGM12)+(1<<CS11)
		out	TCCR1B,tmp1

		; Timer/Counter Interrupt Mask Register (TIMSK), vgl. S.139:
		in	tmp1,TIMSK
		sbr	tmp1,(1<<OCIE1A)
		out	TIMSK,tmp1

		out	SREG,SREG_save	; SREG (und damit das alte I-Flag) 
						;   wiederherstellen
		ret

TC1_Ende:	in	SREG_save,SREG	; SREG (und damit das I-Flag) sichern
		cli				; Interrupts deaktivieren (SREG: I-Flag=0)
		
		in	tmp1,TCCR1B		; Taktsignal ausschalten (vgl. S.136)
		cbr	tmp1,(1<<CS11)	; 
		out	TCCR1B,tmp1		;
		
		in	tmp1,TIMSK		; TC1 Output Compare A Match deaktivieren
		cbr	tmp1,(1<<OCIE1A)	;
		out	TIMSK,tmp1		;
		
		out	SREG,SREG_save	; SREG (und damit das alte I-Flag) 
						;   wiederherstellen
		ret


;*******************************************************************************
;*                                                                             *
;* Interruptroutine: "TIM1_COMPA"  (Timer1, Output Compare A Match Interrupt)  *
;*                                                                             *
;* Zweck: Dieser Interrupt dient der genauen zeitlichen Steuerung der          *
;*        Modulation mit dem DDS.                                              *
;*        Dieser Interrupt wird ausgefhrt, sobald der Zhlerstand des TC1 mit *
;*        dem Inhalt des Output Compare Register 1 A (OCR1AH und OCR1AL)       *
;*        bereinstimmt.                                                       *
;*        Der Timer/Counter1 wird im CTC-Mode (Clear Timer on Compare match)   *
;*        betrieben und mu daher nicht softwaremig zurckgesetzt werden.    *
;*                                                                             *
;*******************************************************************************

TIM1_COMPA:	in	SREG_save,SREG	; SREG (und damit das I-Flag) sichern
		cli				; Interrupts deaktivieren (SREG: I-Flag=0)
		nop	; Programm gekrzt !!!
Int_Ende:	out	SREG,SREG_save	; SREG (und damit das alte I-Flag) 
						;   wiederherstellen
		reti
